# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRDEGIBBS_METADATA = Metadata(
    id="832d935b5fa8fdff97b78ecd355cb0d26be8033a.boutiques",
    name="mrdegibbs",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrdegibbsConfigParamsDictNoTag = typing.TypedDict('_MrdegibbsConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrdegibbsConfigParamsDictTagged = typing.TypedDict('MrdegibbsConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrdegibbsConfigParamsDict = _MrdegibbsConfigParamsDictNoTag | MrdegibbsConfigParamsDictTagged


_MrdegibbsParamsDictNoTag = typing.TypedDict('_MrdegibbsParamsDictNoTag', {
    "axes": typing.NotRequired[list[int] | None],
    "nshifts": typing.NotRequired[int | None],
    "minW": typing.NotRequired[int | None],
    "maxW": typing.NotRequired[int | None],
    "datatype": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrdegibbsConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in": InputPathType,
    "out": str,
})
MrdegibbsParamsDictTagged = typing.TypedDict('MrdegibbsParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrdegibbs"],
    "axes": typing.NotRequired[list[int] | None],
    "nshifts": typing.NotRequired[int | None],
    "minW": typing.NotRequired[int | None],
    "maxW": typing.NotRequired[int | None],
    "datatype": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrdegibbsConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in": InputPathType,
    "out": str,
})
MrdegibbsParamsDict = _MrdegibbsParamsDictNoTag | MrdegibbsParamsDictTagged


def mrdegibbs_config(
    key: str,
    value: str,
) -> MrdegibbsConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrdegibbs_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrdegibbsConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrdegibbs_config_cargs(
    params: MrdegibbsConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrdegibbsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrdegibbsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out: OutputPathType
    """the output image."""


def mrdegibbs_params(
    in_: InputPathType,
    out: str,
    axes: list[int] | None = None,
    nshifts: int | None = None,
    min_w: int | None = None,
    max_w: int | None = None,
    datatype: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrdegibbsConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrdegibbsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        in_: the input image.
        out: the output image.
        axes: select the slice axes (default: 0,1 - i.e. x-y).
        nshifts: discretization of subpixel spacing (default: 20).
        min_w: left border of window used for TV computation (default: 1).
        max_w: right border of window used for TV computation (default: 3).
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrdegibbs",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "in": in_,
        "out": out,
    }
    if axes is not None:
        params["axes"] = axes
    if nshifts is not None:
        params["nshifts"] = nshifts
    if min_w is not None:
        params["minW"] = min_w
    if max_w is not None:
        params["maxW"] = max_w
    if datatype is not None:
        params["datatype"] = datatype
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrdegibbs_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrdegibbsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("axes", None) is not None:
        if not isinstance(params["axes"], list):
            raise StyxValidationError(f'`axes` has the wrong type: Received `{type(params.get("axes", None))}` expected `list[int] | None`')
        for e in params["axes"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`axes` has the wrong type: Received `{type(params.get("axes", None))}` expected `list[int] | None`')
    if params.get("nshifts", None) is not None:
        if not isinstance(params["nshifts"], int):
            raise StyxValidationError(f'`nshifts` has the wrong type: Received `{type(params.get("nshifts", None))}` expected `int | None`')
    if params.get("minW", None) is not None:
        if not isinstance(params["minW"], int):
            raise StyxValidationError(f'`minW` has the wrong type: Received `{type(params.get("minW", None))}` expected `int | None`')
    if params.get("maxW", None) is not None:
        if not isinstance(params["maxW"], int):
            raise StyxValidationError(f'`maxW` has the wrong type: Received `{type(params.get("maxW", None))}` expected `int | None`')
    if params.get("datatype", None) is not None:
        if not isinstance(params["datatype"], str):
            raise StyxValidationError(f'`datatype` has the wrong type: Received `{type(params.get("datatype", None))}` expected `str | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrdegibbsConfigParamsDict] | None`')
        for e in params["config"]:
            mrdegibbs_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("in", None) is None:
        raise StyxValidationError("`in` must not be None")
    if not isinstance(params["in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`in` has the wrong type: Received `{type(params.get("in", None))}` expected `InputPathType`')
    if params.get("out", None) is None:
        raise StyxValidationError("`out` must not be None")
    if not isinstance(params["out"], str):
        raise StyxValidationError(f'`out` has the wrong type: Received `{type(params.get("out", None))}` expected `str`')


def mrdegibbs_cargs(
    params: MrdegibbsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrdegibbs")
    if params.get("axes", None) is not None:
        cargs.extend([
            "-axes",
            *map(str, params.get("axes", None))
        ])
    if params.get("nshifts", None) is not None:
        cargs.extend([
            "-nshifts",
            str(params.get("nshifts", None))
        ])
    if params.get("minW", None) is not None:
        cargs.extend([
            "-minW",
            str(params.get("minW", None))
        ])
    if params.get("maxW", None) is not None:
        cargs.extend([
            "-maxW",
            str(params.get("maxW", None))
        ])
    if params.get("datatype", None) is not None:
        cargs.extend([
            "-datatype",
            params.get("datatype", None)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrdegibbs_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("in", None)))
    cargs.append(params.get("out", None))
    return cargs


def mrdegibbs_outputs(
    params: MrdegibbsParamsDict,
    execution: Execution,
) -> MrdegibbsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrdegibbsOutputs(
        root=execution.output_file("."),
        out=execution.output_file(params.get("out", None)),
    )
    return ret


def mrdegibbs_execute(
    params: MrdegibbsParamsDict,
    runner: Runner | None = None,
) -> MrdegibbsOutputs:
    """
    mrdegibbs
    
    Remove Gibbs Ringing Artifacts.
    
    This application attempts to remove Gibbs ringing artefacts from MRI images
    using the method of local subvoxel-shifts proposed by Kellner et al. (see
    reference below for details).
    
    This command is designed to run on data directly after it has been
    reconstructed by the scanner, before any interpolation of any kind has taken
    place. You should not run this command after any form of motion correction
    (e.g. not after dwifslpreproc). Similarly, if you intend running dwidenoise,
    you should run denoising before this command to not alter the noise
    structure, which would impact on dwidenoise's performance.
    
    Note that this method is designed to work on images acquired with full
    k-space coverage. Running this method on partial Fourier ('half-scan') data
    may lead to suboptimal and/or biased results, as noted in the original
    reference below. There is currently no means of dealing with this; users
    should exercise caution when using this method on partial Fourier data, and
    inspect its output for any obvious artefacts.
    
    References:
    
    Kellner, E; Dhital, B; Kiselev, V.G & Reisert, M. Gibbs-ringing artifact
    removal based on local subvoxel-shifts. Magnetic Resonance in Medicine,
    2016, 76, 1574â€“1581.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrdegibbsOutputs`).
    """
    mrdegibbs_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRDEGIBBS_METADATA)
    params = execution.params(params)
    cargs = mrdegibbs_cargs(params, execution)
    ret = mrdegibbs_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrdegibbs(
    in_: InputPathType,
    out: str,
    axes: list[int] | None = None,
    nshifts: int | None = None,
    min_w: int | None = None,
    max_w: int | None = None,
    datatype: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrdegibbsConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrdegibbsOutputs:
    """
    mrdegibbs
    
    Remove Gibbs Ringing Artifacts.
    
    This application attempts to remove Gibbs ringing artefacts from MRI images
    using the method of local subvoxel-shifts proposed by Kellner et al. (see
    reference below for details).
    
    This command is designed to run on data directly after it has been
    reconstructed by the scanner, before any interpolation of any kind has taken
    place. You should not run this command after any form of motion correction
    (e.g. not after dwifslpreproc). Similarly, if you intend running dwidenoise,
    you should run denoising before this command to not alter the noise
    structure, which would impact on dwidenoise's performance.
    
    Note that this method is designed to work on images acquired with full
    k-space coverage. Running this method on partial Fourier ('half-scan') data
    may lead to suboptimal and/or biased results, as noted in the original
    reference below. There is currently no means of dealing with this; users
    should exercise caution when using this method on partial Fourier data, and
    inspect its output for any obvious artefacts.
    
    References:
    
    Kellner, E; Dhital, B; Kiselev, V.G & Reisert, M. Gibbs-ringing artifact
    removal based on local subvoxel-shifts. Magnetic Resonance in Medicine,
    2016, 76, 1574â€“1581.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        in_: the input image.
        out: the output image.
        axes: select the slice axes (default: 0,1 - i.e. x-y).
        nshifts: discretization of subpixel spacing (default: 20).
        min_w: left border of window used for TV computation (default: 1).
        max_w: right border of window used for TV computation (default: 3).
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrdegibbsOutputs`).
    """
    params = mrdegibbs_params(
        axes=axes,
        nshifts=nshifts,
        min_w=min_w,
        max_w=max_w,
        datatype=datatype,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        in_=in_,
        out=out,
    )
    return mrdegibbs_execute(params, runner)


__all__ = [
    "MRDEGIBBS_METADATA",
    "MrdegibbsConfigParamsDict",
    "MrdegibbsConfigParamsDictTagged",
    "MrdegibbsOutputs",
    "MrdegibbsParamsDict",
    "MrdegibbsParamsDictTagged",
    "mrdegibbs",
    "mrdegibbs_config",
    "mrdegibbs_execute",
    "mrdegibbs_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DIRSTAT_METADATA = Metadata(
    id="369098dbdbd016bf94682e0106a2f2e1711f2a5b.boutiques",
    name="dirstat",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_DirstatFslgradParamsDictNoTag = typing.TypedDict('_DirstatFslgradParamsDictNoTag', {
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
DirstatFslgradParamsDictTagged = typing.TypedDict('DirstatFslgradParamsDictTagged', {
    "@type": typing.Literal["fslgrad"],
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
DirstatFslgradParamsDict = _DirstatFslgradParamsDictNoTag | DirstatFslgradParamsDictTagged


_DirstatConfigParamsDictNoTag = typing.TypedDict('_DirstatConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
DirstatConfigParamsDictTagged = typing.TypedDict('DirstatConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
DirstatConfigParamsDict = _DirstatConfigParamsDictNoTag | DirstatConfigParamsDictTagged


_DirstatParamsDictNoTag = typing.TypedDict('_DirstatParamsDictNoTag', {
    "output": typing.NotRequired[str | None],
    "shells": typing.NotRequired[list[float] | None],
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[DirstatFslgradParamsDict | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DirstatConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "dirs": InputPathType,
})
DirstatParamsDictTagged = typing.TypedDict('DirstatParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dirstat"],
    "output": typing.NotRequired[str | None],
    "shells": typing.NotRequired[list[float] | None],
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[DirstatFslgradParamsDict | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DirstatConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "dirs": InputPathType,
})
DirstatParamsDict = _DirstatParamsDictNoTag | DirstatParamsDictTagged


def dirstat_fslgrad(
    bvecs: InputPathType,
    bvals: InputPathType,
) -> DirstatFslgradParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvals: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fslgrad",
        "bvecs": bvecs,
        "bvals": bvals,
    }
    return params


def dirstat_fslgrad_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DirstatFslgradParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs", None) is None:
        raise StyxValidationError("`bvecs` must not be None")
    if not isinstance(params["bvecs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvecs` has the wrong type: Received `{type(params.get("bvecs", None))}` expected `InputPathType`')
    if params.get("bvals", None) is None:
        raise StyxValidationError("`bvals` must not be None")
    if not isinstance(params["bvals"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvals` has the wrong type: Received `{type(params.get("bvals", None))}` expected `InputPathType`')


def dirstat_fslgrad_cargs(
    params: DirstatFslgradParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fslgrad")
    cargs.append(execution.input_file(params.get("bvecs", None)))
    cargs.append(execution.input_file(params.get("bvals", None)))
    return cargs


def dirstat_config(
    key: str,
    value: str,
) -> DirstatConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dirstat_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DirstatConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dirstat_config_cargs(
    params: DirstatConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class DirstatOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DirstatParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def dirstat_params(
    dirs: InputPathType,
    output: str | None = None,
    shells: list[float] | None = None,
    grad: InputPathType | None = None,
    fslgrad: DirstatFslgradParamsDict | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DirstatConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> DirstatParamsDictTagged:
    """
    Build parameters.
    
    Args:
        dirs: the text file or image containing the directions.
        output: output selected metrics as a space-delimited list, suitable for\
            use in scripts. This will produce one line of values per selected\
            shell. Valid metrics are as specified in the description above.
        shells: specify one or more b-values to use during processing, as a\
            comma-separated list of the desired approximate b-values (b-values are\
            clustered to allow for small deviations). Note that some commands are\
            incompatible with multiple b-values, and will report an error if more\
            than one b-value is provided.\
            WARNING: note that, even though the b=0 volumes are never referred\
            to as shells in the literature, they still have to be explicitly\
            included in the list of b-values as provided to the -shell option!\
            Several algorithms which include the b=0 volumes in their\
            computations may otherwise return an undesired result.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dirstat",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "dirs": dirs,
    }
    if output is not None:
        params["output"] = output
    if shells is not None:
        params["shells"] = shells
    if grad is not None:
        params["grad"] = grad
    if fslgrad is not None:
        params["fslgrad"] = fslgrad
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dirstat_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DirstatParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("output", None) is not None:
        if not isinstance(params["output"], str):
            raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str | None`')
    if params.get("shells", None) is not None:
        if not isinstance(params["shells"], list):
            raise StyxValidationError(f'`shells` has the wrong type: Received `{type(params.get("shells", None))}` expected `list[float] | None`')
        for e in params["shells"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`shells` has the wrong type: Received `{type(params.get("shells", None))}` expected `list[float] | None`')
    if params.get("grad", None) is not None:
        if not isinstance(params["grad"], (pathlib.Path, str)):
            raise StyxValidationError(f'`grad` has the wrong type: Received `{type(params.get("grad", None))}` expected `InputPathType | None`')
    if params.get("fslgrad", None) is not None:
        dirstat_fslgrad_validate(params["fslgrad"])
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[DirstatConfigParamsDict] | None`')
        for e in params["config"]:
            dirstat_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("dirs", None) is None:
        raise StyxValidationError("`dirs` must not be None")
    if not isinstance(params["dirs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`dirs` has the wrong type: Received `{type(params.get("dirs", None))}` expected `InputPathType`')


def dirstat_cargs(
    params: DirstatParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dirstat")
    if params.get("output", None) is not None:
        cargs.extend([
            "-output",
            params.get("output", None)
        ])
    if params.get("shells", None) is not None:
        cargs.extend([
            "-shells",
            ",".join(map(str, params.get("shells", None)))
        ])
    if params.get("grad", None) is not None:
        cargs.extend([
            "-grad",
            execution.input_file(params.get("grad", None))
        ])
    if params.get("fslgrad", None) is not None:
        cargs.extend(dirstat_fslgrad_cargs(params.get("fslgrad", None), execution))
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dirstat_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("dirs", None)))
    return cargs


def dirstat_outputs(
    params: DirstatParamsDict,
    execution: Execution,
) -> DirstatOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DirstatOutputs(
        root=execution.output_file("."),
    )
    return ret


def dirstat_execute(
    params: DirstatParamsDict,
    runner: Runner | None = None,
) -> DirstatOutputs:
    """
    dirstat
    
    Report statistics on a direction set.
    
    This command will accept as inputs:
    
    - directions file in spherical coordinates (ASCII text, [ az el ]
    space-separated values, one per line);
    
    - directions file in Cartesian coordinates (ASCII text, [ x y z ]
    space-separated values, one per line);
    
    - DW gradient files (MRtrix format: ASCII text, [ x y z b ] space-separated
    values, one per line);
    
    - image files, using the DW gradient scheme found in the header (or provided
    using the appropriate command line options below).
    
    By default, this produces all relevant metrics for the direction set
    provided. If the direction set contains multiple shells, metrics are
    provided for each shell separately.
    
    Metrics are produced assuming a unipolar or bipolar electrostatic repulsion
    model, producing the potential energy (total, mean, min & max), and the
    nearest-neighbour angles (mean, min & max). The condition number is also
    produced for the spherical harmonic fits up to the highest harmonic order
    supported by the number of volumes. Finally, the norm of the mean direction
    vector is provided as a measure of the overall symmetry of the direction set
    (important with respect to eddy-current resilience).
    
    Specific metrics can also be queried independently via the "-output" option,
    using these shorthands:
    U/B for unipolar/bipolar model,
    E/N for energy and nearest-neighbour respectively,
    t/-/+ for total/min/max respectively (mean implied otherwise);
    SHn for condition number of SH fit at order n (with n an even integer);
    ASYM for asymmetry index (norm of mean direction vector);
    N for the number of directions.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DirstatOutputs`).
    """
    dirstat_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DIRSTAT_METADATA)
    params = execution.params(params)
    cargs = dirstat_cargs(params, execution)
    ret = dirstat_outputs(params, execution)
    execution.run(cargs)
    return ret


def dirstat(
    dirs: InputPathType,
    output: str | None = None,
    shells: list[float] | None = None,
    grad: InputPathType | None = None,
    fslgrad: DirstatFslgradParamsDict | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DirstatConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DirstatOutputs:
    """
    dirstat
    
    Report statistics on a direction set.
    
    This command will accept as inputs:
    
    - directions file in spherical coordinates (ASCII text, [ az el ]
    space-separated values, one per line);
    
    - directions file in Cartesian coordinates (ASCII text, [ x y z ]
    space-separated values, one per line);
    
    - DW gradient files (MRtrix format: ASCII text, [ x y z b ] space-separated
    values, one per line);
    
    - image files, using the DW gradient scheme found in the header (or provided
    using the appropriate command line options below).
    
    By default, this produces all relevant metrics for the direction set
    provided. If the direction set contains multiple shells, metrics are
    provided for each shell separately.
    
    Metrics are produced assuming a unipolar or bipolar electrostatic repulsion
    model, producing the potential energy (total, mean, min & max), and the
    nearest-neighbour angles (mean, min & max). The condition number is also
    produced for the spherical harmonic fits up to the highest harmonic order
    supported by the number of volumes. Finally, the norm of the mean direction
    vector is provided as a measure of the overall symmetry of the direction set
    (important with respect to eddy-current resilience).
    
    Specific metrics can also be queried independently via the "-output" option,
    using these shorthands:
    U/B for unipolar/bipolar model,
    E/N for energy and nearest-neighbour respectively,
    t/-/+ for total/min/max respectively (mean implied otherwise);
    SHn for condition number of SH fit at order n (with n an even integer);
    ASYM for asymmetry index (norm of mean direction vector);
    N for the number of directions.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        dirs: the text file or image containing the directions.
        output: output selected metrics as a space-delimited list, suitable for\
            use in scripts. This will produce one line of values per selected\
            shell. Valid metrics are as specified in the description above.
        shells: specify one or more b-values to use during processing, as a\
            comma-separated list of the desired approximate b-values (b-values are\
            clustered to allow for small deviations). Note that some commands are\
            incompatible with multiple b-values, and will report an error if more\
            than one b-value is provided.\
            WARNING: note that, even though the b=0 volumes are never referred\
            to as shells in the literature, they still have to be explicitly\
            included in the list of b-values as provided to the -shell option!\
            Several algorithms which include the b=0 volumes in their\
            computations may otherwise return an undesired result.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DirstatOutputs`).
    """
    params = dirstat_params(
        output=output,
        shells=shells,
        grad=grad,
        fslgrad=fslgrad,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        dirs=dirs,
    )
    return dirstat_execute(params, runner)


__all__ = [
    "DIRSTAT_METADATA",
    "DirstatConfigParamsDict",
    "DirstatConfigParamsDictTagged",
    "DirstatFslgradParamsDict",
    "DirstatFslgradParamsDictTagged",
    "DirstatOutputs",
    "DirstatParamsDict",
    "DirstatParamsDictTagged",
    "dirstat",
    "dirstat_config",
    "dirstat_execute",
    "dirstat_fslgrad",
    "dirstat_params",
]



"""
MRTrix3

MRtrix3 provides a set of tools to perform various types of diffusion MRI
analyses, from various forms of tractography through to next-generation
group-level analyses. It is designed with consistency, performance, and
stability in mind, and is freely available under an open-source license. It is
developed and maintained by a team of experts in the field, fostering an active
community of users from diverse backgrounds.

Author: MRTrix3 Developers

URL: https://www.mrtrix.org/
"""
# This file was auto generated by Styx.
# Do not edit this file directly.

from .afdconnectivity import *
from .amp2response import *
from .amp2sh import *
from .connectome2tck import *
from .connectomeedit import *
from .connectomestats import *
from .dcmedit import *
from .dcminfo import *
from .dirflip import *
from .dirgen import *
from .dirmerge import *
from .dirorder import *
from .dirsplit import *
from .dirstat import *
from .dwi2adc import *
from .dwi2fod import *
from .dwi2mask import *
from .dwi2response import *
from .dwi2tensor import *
from .dwibiascorrect import *
from .dwidenoise import *
from .dwiextract import *
from .dwigradcheck import *
from .fixel2peaks import *
from .fixel2sh import *
from .fixel2tsf import *
from .fixel2voxel import *
from .fixelcfestats import *
from .fixelconnectivity import *
from .fixelconvert import *
from .fixelcorrespondence import *
from .fixelcrop import *
from .fixelfilter import *
from .fixelreorient import *
from .fod2dec import *
from .fod2fixel import *
from .label2colour import *
from .label2mesh import *
from .labelconvert import *
from .labelstats import *
from .maskdump import *
from .maskfilter import *
from .mesh2voxel import *
from .meshconvert import *
from .meshfilter import *
from .mraverageheader import *
from .mrcalc import *
from .mrcat import *
from .mrcentroid import *
from .mrcheckerboardmask import *
from .mrclusterstats import *
from .mrcolour import *
from .mrconvert import *
from .mrdegibbs import *
from .mrdump import *
from .mredit import *
from .mrfilter import *
from .mrgrid import *
from .mrhistmatch import *
from .mrhistogram import *
from .mrinfo import *
from .mrmath import *
from .mrmetric import *
from .mrregister import *
from .mrstats import *
from .mrthreshold import *
from .mrtransform import *
from .mtnormalise import *
from .peaks2amp import *
from .peaks2fixel import *
from .responsemean import *
from .sh2amp import *
from .sh2peaks import *
from .sh2power import *
from .sh2response import *
from .shbasis import *
from .shconv import *
from .tck2connectome import *
from .tck2fixel import *
from .tckconvert import *
from .tckdfc import *
from .tckedit import *
from .tckgen import *
from .tckglobal import *
from .tckinfo import *
from .tckmap import *
from .tckresample import *
from .tcksample import *
from .tcksift import *
from .tcksift2 import *
from .tckstats import *
from .tcktransform import *
from .tensor2metric import *
from .transformcalc import *
from .transformcompose import *
from .transformconvert import *
from .tsfdivide import *
from .tsfinfo import *
from .tsfmult import *
from .tsfsmooth import *
from .tsfthreshold import *
from .tsfvalidate import *
from .v_5tt2gmwmi import *
from .v_5tt2vis import *
from .v_5ttcheck import *
from .v_5ttedit import *
from .v_5ttgen import *
from .vectorstats import *
from .voxel2fixel import *
from .voxel2mesh import *
from .warp2metric import *
from .warpconvert import *
from .warpcorrect import *
from .warpinit import *
from .warpinvert import *
from styxdefs import Runner as _Runner


def execute(
    params: dict,
    runner: _Runner | None = None,
) -> None:
    """
    Run a command in this package dynamically from a params object.
    
    Args:
        params: The parameters.
        runner: Command runner.
    """
    return {
        "mrtrix/5tt2gmwmi": v_5tt2gmwmi_execute,
        "mrtrix/5tt2vis": v_5tt2vis_execute,
        "mrtrix/5ttcheck": v_5ttcheck_execute,
        "mrtrix/5ttedit": v_5ttedit_execute,
        "mrtrix/5ttgen": v_5ttgen_execute,
        "mrtrix/afdconnectivity": afdconnectivity_execute,
        "mrtrix/amp2response": amp2response_execute,
        "mrtrix/amp2sh": amp2sh_execute,
        "mrtrix/connectome2tck": connectome2tck_execute,
        "mrtrix/connectomeedit": connectomeedit_execute,
        "mrtrix/connectomestats": connectomestats_execute,
        "mrtrix/dcmedit": dcmedit_execute,
        "mrtrix/dcminfo": dcminfo_execute,
        "mrtrix/dirflip": dirflip_execute,
        "mrtrix/dirgen": dirgen_execute,
        "mrtrix/dirmerge": dirmerge_execute,
        "mrtrix/dirorder": dirorder_execute,
        "mrtrix/dirsplit": dirsplit_execute,
        "mrtrix/dirstat": dirstat_execute,
        "mrtrix/dwi2adc": dwi2adc_execute,
        "mrtrix/dwi2fod": dwi2fod_execute,
        "mrtrix/dwi2mask": dwi2mask_execute,
        "mrtrix/dwi2response": dwi2response_execute,
        "mrtrix/dwi2tensor": dwi2tensor_execute,
        "mrtrix/dwibiascorrect": dwibiascorrect_execute,
        "mrtrix/dwidenoise": dwidenoise_execute,
        "mrtrix/dwiextract": dwiextract_execute,
        "mrtrix/dwigradcheck": dwigradcheck_execute,
        "mrtrix/fixel2peaks": fixel2peaks_execute,
        "mrtrix/fixel2sh": fixel2sh_execute,
        "mrtrix/fixel2tsf": fixel2tsf_execute,
        "mrtrix/fixel2voxel": fixel2voxel_execute,
        "mrtrix/fixelcfestats": fixelcfestats_execute,
        "mrtrix/fixelconnectivity": fixelconnectivity_execute,
        "mrtrix/fixelconvert": fixelconvert_execute,
        "mrtrix/fixelcorrespondence": fixelcorrespondence_execute,
        "mrtrix/fixelcrop": fixelcrop_execute,
        "mrtrix/fixelfilter": fixelfilter_execute,
        "mrtrix/fixelreorient": fixelreorient_execute,
        "mrtrix/fod2dec": fod2dec_execute,
        "mrtrix/fod2fixel": fod2fixel_execute,
        "mrtrix/label2colour": label2colour_execute,
        "mrtrix/label2mesh": label2mesh_execute,
        "mrtrix/labelconvert": labelconvert_execute,
        "mrtrix/labelstats": labelstats_execute,
        "mrtrix/maskdump": maskdump_execute,
        "mrtrix/maskfilter": maskfilter_execute,
        "mrtrix/mesh2voxel": mesh2voxel_execute,
        "mrtrix/meshconvert": meshconvert_execute,
        "mrtrix/meshfilter": meshfilter_execute,
        "mrtrix/mraverageheader": mraverageheader_execute,
        "mrtrix/mrcalc": mrcalc_execute,
        "mrtrix/mrcat": mrcat_execute,
        "mrtrix/mrcentroid": mrcentroid_execute,
        "mrtrix/mrcheckerboardmask": mrcheckerboardmask_execute,
        "mrtrix/mrclusterstats": mrclusterstats_execute,
        "mrtrix/mrcolour": mrcolour_execute,
        "mrtrix/mrconvert": mrconvert_execute,
        "mrtrix/mrdegibbs": mrdegibbs_execute,
        "mrtrix/mrdump": mrdump_execute,
        "mrtrix/mredit": mredit_execute,
        "mrtrix/mrfilter": mrfilter_execute,
        "mrtrix/mrgrid": mrgrid_execute,
        "mrtrix/mrhistmatch": mrhistmatch_execute,
        "mrtrix/mrhistogram": mrhistogram_execute,
        "mrtrix/mrinfo": mrinfo_execute,
        "mrtrix/mrmath": mrmath_execute,
        "mrtrix/mrmetric": mrmetric_execute,
        "mrtrix/mrregister": mrregister_execute,
        "mrtrix/mrstats": mrstats_execute,
        "mrtrix/mrthreshold": mrthreshold_execute,
        "mrtrix/mrtransform": mrtransform_execute,
        "mrtrix/mtnormalise": mtnormalise_execute,
        "mrtrix/peaks2amp": peaks2amp_execute,
        "mrtrix/peaks2fixel": peaks2fixel_execute,
        "mrtrix/responsemean": responsemean_execute,
        "mrtrix/sh2amp": sh2amp_execute,
        "mrtrix/sh2peaks": sh2peaks_execute,
        "mrtrix/sh2power": sh2power_execute,
        "mrtrix/sh2response": sh2response_execute,
        "mrtrix/shbasis": shbasis_execute,
        "mrtrix/shconv": shconv_execute,
        "mrtrix/tck2connectome": tck2connectome_execute,
        "mrtrix/tck2fixel": tck2fixel_execute,
        "mrtrix/tckconvert": tckconvert_execute,
        "mrtrix/tckdfc": tckdfc_execute,
        "mrtrix/tckedit": tckedit_execute,
        "mrtrix/tckgen": tckgen_execute,
        "mrtrix/tckglobal": tckglobal_execute,
        "mrtrix/tckinfo": tckinfo_execute,
        "mrtrix/tckmap": tckmap_execute,
        "mrtrix/tckresample": tckresample_execute,
        "mrtrix/tcksample": tcksample_execute,
        "mrtrix/tcksift": tcksift_execute,
        "mrtrix/tcksift2": tcksift2_execute,
        "mrtrix/tckstats": tckstats_execute,
        "mrtrix/tcktransform": tcktransform_execute,
        "mrtrix/tensor2metric": tensor2metric_execute,
        "mrtrix/transformcalc": transformcalc_execute,
        "mrtrix/transformcompose": transformcompose_execute,
        "mrtrix/transformconvert": transformconvert_execute,
        "mrtrix/tsfdivide": tsfdivide_execute,
        "mrtrix/tsfinfo": tsfinfo_execute,
        "mrtrix/tsfmult": tsfmult_execute,
        "mrtrix/tsfsmooth": tsfsmooth_execute,
        "mrtrix/tsfthreshold": tsfthreshold_execute,
        "mrtrix/tsfvalidate": tsfvalidate_execute,
        "mrtrix/vectorstats": vectorstats_execute,
        "mrtrix/voxel2fixel": voxel2fixel_execute,
        "mrtrix/voxel2mesh": voxel2mesh_execute,
        "mrtrix/warp2metric": warp2metric_execute,
        "mrtrix/warpconvert": warpconvert_execute,
        "mrtrix/warpcorrect": warpcorrect_execute,
        "mrtrix/warpinit": warpinit_execute,
        "mrtrix/warpinvert": warpinvert_execute,
    }[params["@type"]](params, runner)



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRHISTMATCH_METADATA = Metadata(
    id="1798cd7ea1a79d9523cf1fd1a7ec35b94aa4a5ab.boutiques",
    name="mrhistmatch",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrhistmatchConfigParamsDictNoTag = typing.TypedDict('_MrhistmatchConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrhistmatchConfigParamsDictTagged = typing.TypedDict('MrhistmatchConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrhistmatchConfigParamsDict = _MrhistmatchConfigParamsDictNoTag | MrhistmatchConfigParamsDictTagged


_MrhistmatchParamsDictNoTag = typing.TypedDict('_MrhistmatchParamsDictNoTag', {
    "mask_input": typing.NotRequired[InputPathType | None],
    "mask_target": typing.NotRequired[InputPathType | None],
    "bins": typing.NotRequired[int | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrhistmatchConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "type": str,
    "input": InputPathType,
    "target": InputPathType,
    "output": str,
})
MrhistmatchParamsDictTagged = typing.TypedDict('MrhistmatchParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrhistmatch"],
    "mask_input": typing.NotRequired[InputPathType | None],
    "mask_target": typing.NotRequired[InputPathType | None],
    "bins": typing.NotRequired[int | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrhistmatchConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "type": str,
    "input": InputPathType,
    "target": InputPathType,
    "output": str,
})
MrhistmatchParamsDict = _MrhistmatchParamsDictNoTag | MrhistmatchParamsDictTagged


def mrhistmatch_config(
    key: str,
    value: str,
) -> MrhistmatchConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrhistmatch_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrhistmatchConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrhistmatch_config_cargs(
    params: MrhistmatchConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrhistmatchOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrhistmatchParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image"""


def mrhistmatch_params(
    type_: str,
    input_: InputPathType,
    target: InputPathType,
    output: str,
    mask_input: InputPathType | None = None,
    mask_target: InputPathType | None = None,
    bins: int | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrhistmatchConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrhistmatchParamsDictTagged:
    """
    Build parameters.
    
    Args:
        type_: type of histogram matching to perform; options are:\
            scale,linear,nonlinear.
        input_: the input image to be modified.
        target: the input image from which to derive the target histogram.
        output: the output image.
        mask_input: only generate input histogram based on a specified binary\
            mask image.
        mask_target: only generate target histogram based on a specified binary\
            mask image.
        bins: the number of bins to use to generate the histograms.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrhistmatch",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "type": type_,
        "input": input_,
        "target": target,
        "output": output,
    }
    if mask_input is not None:
        params["mask_input"] = mask_input
    if mask_target is not None:
        params["mask_target"] = mask_target
    if bins is not None:
        params["bins"] = bins
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrhistmatch_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrhistmatchParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("mask_input", None) is not None:
        if not isinstance(params["mask_input"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask_input` has the wrong type: Received `{type(params.get("mask_input", None))}` expected `InputPathType | None`')
    if params.get("mask_target", None) is not None:
        if not isinstance(params["mask_target"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask_target` has the wrong type: Received `{type(params.get("mask_target", None))}` expected `InputPathType | None`')
    if params.get("bins", None) is not None:
        if not isinstance(params["bins"], int):
            raise StyxValidationError(f'`bins` has the wrong type: Received `{type(params.get("bins", None))}` expected `int | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrhistmatchConfigParamsDict] | None`')
        for e in params["config"]:
            mrhistmatch_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("type", None) is None:
        raise StyxValidationError("`type` must not be None")
    if not isinstance(params["type"], str):
        raise StyxValidationError(f'`type` has the wrong type: Received `{type(params.get("type", None))}` expected `str`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("target", None) is None:
        raise StyxValidationError("`target` must not be None")
    if not isinstance(params["target"], (pathlib.Path, str)):
        raise StyxValidationError(f'`target` has the wrong type: Received `{type(params.get("target", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def mrhistmatch_cargs(
    params: MrhistmatchParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrhistmatch")
    if params.get("mask_input", None) is not None:
        cargs.extend([
            "-mask_input",
            execution.input_file(params.get("mask_input", None))
        ])
    if params.get("mask_target", None) is not None:
        cargs.extend([
            "-mask_target",
            execution.input_file(params.get("mask_target", None))
        ])
    if params.get("bins", None) is not None:
        cargs.extend([
            "-bins",
            str(params.get("bins", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrhistmatch_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(params.get("type", None))
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(execution.input_file(params.get("target", None)))
    cargs.append(params.get("output", None))
    return cargs


def mrhistmatch_outputs(
    params: MrhistmatchParamsDict,
    execution: Execution,
) -> MrhistmatchOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrhistmatchOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def mrhistmatch_execute(
    params: MrhistmatchParamsDict,
    runner: Runner | None = None,
) -> MrhistmatchOutputs:
    """
    mrhistmatch
    
    Modify the intensities of one image to match the histogram of another.
    
    
    
    References:
    
    * If using inverse contrast normalization for inter-modal (DWI - T1)
    registration:
    Bhushan, C.; Haldar, J. P.; Choi, S.; Joshi, A. A.; Shattuck, D. W. & Leahy,
    R. M. Co-registration and distortion correction of diffusion and anatomical
    images based on inverse contrast normalization. NeuroImage, 2015, 115,
    269-280.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrhistmatchOutputs`).
    """
    mrhistmatch_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRHISTMATCH_METADATA)
    params = execution.params(params)
    cargs = mrhistmatch_cargs(params, execution)
    ret = mrhistmatch_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrhistmatch(
    type_: str,
    input_: InputPathType,
    target: InputPathType,
    output: str,
    mask_input: InputPathType | None = None,
    mask_target: InputPathType | None = None,
    bins: int | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrhistmatchConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrhistmatchOutputs:
    """
    mrhistmatch
    
    Modify the intensities of one image to match the histogram of another.
    
    
    
    References:
    
    * If using inverse contrast normalization for inter-modal (DWI - T1)
    registration:
    Bhushan, C.; Haldar, J. P.; Choi, S.; Joshi, A. A.; Shattuck, D. W. & Leahy,
    R. M. Co-registration and distortion correction of diffusion and anatomical
    images based on inverse contrast normalization. NeuroImage, 2015, 115,
    269-280.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        type_: type of histogram matching to perform; options are:\
            scale,linear,nonlinear.
        input_: the input image to be modified.
        target: the input image from which to derive the target histogram.
        output: the output image.
        mask_input: only generate input histogram based on a specified binary\
            mask image.
        mask_target: only generate target histogram based on a specified binary\
            mask image.
        bins: the number of bins to use to generate the histograms.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrhistmatchOutputs`).
    """
    params = mrhistmatch_params(
        mask_input=mask_input,
        mask_target=mask_target,
        bins=bins,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        type_=type_,
        input_=input_,
        target=target,
        output=output,
    )
    return mrhistmatch_execute(params, runner)


__all__ = [
    "MRHISTMATCH_METADATA",
    "MrhistmatchConfigParamsDict",
    "MrhistmatchConfigParamsDictTagged",
    "MrhistmatchOutputs",
    "MrhistmatchParamsDict",
    "MrhistmatchParamsDictTagged",
    "mrhistmatch",
    "mrhistmatch_config",
    "mrhistmatch_execute",
    "mrhistmatch_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DWIEXTRACT_METADATA = Metadata(
    id="0021ca6ba5d653dd2d95adc256257bec0ea68206.boutiques",
    name="dwiextract",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_DwiextractFslgradParamsDictNoTag = typing.TypedDict('_DwiextractFslgradParamsDictNoTag', {
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
DwiextractFslgradParamsDictTagged = typing.TypedDict('DwiextractFslgradParamsDictTagged', {
    "@type": typing.Literal["fslgrad"],
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
DwiextractFslgradParamsDict = _DwiextractFslgradParamsDictNoTag | DwiextractFslgradParamsDictTagged


_DwiextractExportGradFslParamsDictNoTag = typing.TypedDict('_DwiextractExportGradFslParamsDictNoTag', {
    "bvecs_path": str,
    "bvals_path": str,
})
DwiextractExportGradFslParamsDictTagged = typing.TypedDict('DwiextractExportGradFslParamsDictTagged', {
    "@type": typing.Literal["export_grad_fsl"],
    "bvecs_path": str,
    "bvals_path": str,
})
DwiextractExportGradFslParamsDict = _DwiextractExportGradFslParamsDictNoTag | DwiextractExportGradFslParamsDictTagged


_DwiextractImportPeEddyParamsDictNoTag = typing.TypedDict('_DwiextractImportPeEddyParamsDictNoTag', {
    "config": InputPathType,
    "indices": InputPathType,
})
DwiextractImportPeEddyParamsDictTagged = typing.TypedDict('DwiextractImportPeEddyParamsDictTagged', {
    "@type": typing.Literal["import_pe_eddy"],
    "config": InputPathType,
    "indices": InputPathType,
})
DwiextractImportPeEddyParamsDict = _DwiextractImportPeEddyParamsDictNoTag | DwiextractImportPeEddyParamsDictTagged


_DwiextractVariousStringParamsDictNoTag = typing.TypedDict('_DwiextractVariousStringParamsDictNoTag', {
    "obj": str,
})
DwiextractVariousStringParamsDictTagged = typing.TypedDict('DwiextractVariousStringParamsDictTagged', {
    "@type": typing.Literal["VariousString"],
    "obj": str,
})
DwiextractVariousStringParamsDict = _DwiextractVariousStringParamsDictNoTag | DwiextractVariousStringParamsDictTagged


_DwiextractVariousFileParamsDictNoTag = typing.TypedDict('_DwiextractVariousFileParamsDictNoTag', {
    "obj": InputPathType,
})
DwiextractVariousFileParamsDictTagged = typing.TypedDict('DwiextractVariousFileParamsDictTagged', {
    "@type": typing.Literal["VariousFile"],
    "obj": InputPathType,
})
DwiextractVariousFileParamsDict = _DwiextractVariousFileParamsDictNoTag | DwiextractVariousFileParamsDictTagged


_DwiextractConfigParamsDictNoTag = typing.TypedDict('_DwiextractConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
DwiextractConfigParamsDictTagged = typing.TypedDict('DwiextractConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
DwiextractConfigParamsDict = _DwiextractConfigParamsDictNoTag | DwiextractConfigParamsDictTagged


_DwiextractParamsDictNoTag = typing.TypedDict('_DwiextractParamsDictNoTag', {
    "bzero": bool,
    "no_bzero": bool,
    "singleshell": bool,
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[DwiextractFslgradParamsDict | None],
    "shells": typing.NotRequired[list[float] | None],
    "export_grad_mrtrix": typing.NotRequired[str | None],
    "export_grad_fsl": typing.NotRequired[DwiextractExportGradFslParamsDict | None],
    "import_pe_table": typing.NotRequired[InputPathType | None],
    "import_pe_eddy": typing.NotRequired[DwiextractImportPeEddyParamsDict | None],
    "pe": typing.NotRequired[list[float] | None],
    "strides": typing.NotRequired[typing.Union[DwiextractVariousStringParamsDictTagged, DwiextractVariousFileParamsDictTagged] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DwiextractConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
DwiextractParamsDictTagged = typing.TypedDict('DwiextractParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dwiextract"],
    "bzero": bool,
    "no_bzero": bool,
    "singleshell": bool,
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[DwiextractFslgradParamsDict | None],
    "shells": typing.NotRequired[list[float] | None],
    "export_grad_mrtrix": typing.NotRequired[str | None],
    "export_grad_fsl": typing.NotRequired[DwiextractExportGradFslParamsDict | None],
    "import_pe_table": typing.NotRequired[InputPathType | None],
    "import_pe_eddy": typing.NotRequired[DwiextractImportPeEddyParamsDict | None],
    "pe": typing.NotRequired[list[float] | None],
    "strides": typing.NotRequired[typing.Union[DwiextractVariousStringParamsDictTagged, DwiextractVariousFileParamsDictTagged] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DwiextractConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
DwiextractParamsDict = _DwiextractParamsDictNoTag | DwiextractParamsDictTagged


def dwiextract_strides_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString": dwiextract_various_string_cargs,
        "VariousFile": dwiextract_various_file_cargs,
    }.get(t)


def dwiextract_strides_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString": dwiextract_various_string_validate,
        "VariousFile": dwiextract_various_file_validate,
    }.get(t)


def dwiextract_fslgrad(
    bvecs: InputPathType,
    bvals: InputPathType,
) -> DwiextractFslgradParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvals: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fslgrad",
        "bvecs": bvecs,
        "bvals": bvals,
    }
    return params


def dwiextract_fslgrad_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwiextractFslgradParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs", None) is None:
        raise StyxValidationError("`bvecs` must not be None")
    if not isinstance(params["bvecs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvecs` has the wrong type: Received `{type(params.get("bvecs", None))}` expected `InputPathType`')
    if params.get("bvals", None) is None:
        raise StyxValidationError("`bvals` must not be None")
    if not isinstance(params["bvals"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvals` has the wrong type: Received `{type(params.get("bvals", None))}` expected `InputPathType`')


def dwiextract_fslgrad_cargs(
    params: DwiextractFslgradParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fslgrad")
    cargs.append(execution.input_file(params.get("bvecs", None)))
    cargs.append(execution.input_file(params.get("bvals", None)))
    return cargs


class DwiextractExportGradFslOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DwiextractExportGradFslParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    bvecs_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    bvals_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""


def dwiextract_export_grad_fsl(
    bvecs_path: str,
    bvals_path: str,
) -> DwiextractExportGradFslParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs_path: export the diffusion-weighted gradient table to files in\
            FSL (bvecs / bvals) format.
        bvals_path: export the diffusion-weighted gradient table to files in\
            FSL (bvecs / bvals) format.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "export_grad_fsl",
        "bvecs_path": bvecs_path,
        "bvals_path": bvals_path,
    }
    return params


def dwiextract_export_grad_fsl_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwiextractExportGradFslParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs_path", None) is None:
        raise StyxValidationError("`bvecs_path` must not be None")
    if not isinstance(params["bvecs_path"], str):
        raise StyxValidationError(f'`bvecs_path` has the wrong type: Received `{type(params.get("bvecs_path", None))}` expected `str`')
    if params.get("bvals_path", None) is None:
        raise StyxValidationError("`bvals_path` must not be None")
    if not isinstance(params["bvals_path"], str):
        raise StyxValidationError(f'`bvals_path` has the wrong type: Received `{type(params.get("bvals_path", None))}` expected `str`')


def dwiextract_export_grad_fsl_cargs(
    params: DwiextractExportGradFslParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-export_grad_fsl")
    cargs.append(params.get("bvecs_path", None))
    cargs.append(params.get("bvals_path", None))
    return cargs


def dwiextract_export_grad_fsl_outputs(
    params: DwiextractExportGradFslParamsDict,
    execution: Execution,
) -> DwiextractExportGradFslOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DwiextractExportGradFslOutputs(
        root=execution.output_file("."),
        bvecs_path=execution.output_file(params.get("bvecs_path", None)),
        bvals_path=execution.output_file(params.get("bvals_path", None)),
    )
    return ret


def dwiextract_import_pe_eddy(
    config: InputPathType,
    indices: InputPathType,
) -> DwiextractImportPeEddyParamsDictTagged:
    """
    Build parameters.
    
    Args:
        config: import phase-encoding information from an EDDY-style config /\
            index file pair.
        indices: import phase-encoding information from an EDDY-style config /\
            index file pair.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "import_pe_eddy",
        "config": config,
        "indices": indices,
    }
    return params


def dwiextract_import_pe_eddy_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwiextractImportPeEddyParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("config", None) is None:
        raise StyxValidationError("`config` must not be None")
    if not isinstance(params["config"], (pathlib.Path, str)):
        raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `InputPathType`')
    if params.get("indices", None) is None:
        raise StyxValidationError("`indices` must not be None")
    if not isinstance(params["indices"], (pathlib.Path, str)):
        raise StyxValidationError(f'`indices` has the wrong type: Received `{type(params.get("indices", None))}` expected `InputPathType`')


def dwiextract_import_pe_eddy_cargs(
    params: DwiextractImportPeEddyParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-import_pe_eddy")
    cargs.append(execution.input_file(params.get("config", None)))
    cargs.append(execution.input_file(params.get("indices", None)))
    return cargs


def dwiextract_various_string(
    obj: str,
) -> DwiextractVariousStringParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString",
        "obj": obj,
    }
    return params


def dwiextract_various_string_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwiextractVariousStringParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def dwiextract_various_string_cargs(
    params: DwiextractVariousStringParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def dwiextract_various_file(
    obj: InputPathType,
) -> DwiextractVariousFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile",
        "obj": obj,
    }
    return params


def dwiextract_various_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwiextractVariousFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def dwiextract_various_file_cargs(
    params: DwiextractVariousFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


def dwiextract_config(
    key: str,
    value: str,
) -> DwiextractConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dwiextract_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwiextractConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dwiextract_config_cargs(
    params: DwiextractConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class DwiextractOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DwiextractParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image (diffusion-weighted volumes by default)."""
    export_grad_mrtrix: OutputPathType | None
    """export the diffusion-weighted gradient table to file in MRtrix format """
    export_grad_fsl: DwiextractExportGradFslOutputs | None
    """Outputs from `dwiextract_export_grad_fsl_outputs`."""


def dwiextract_params(
    input_: InputPathType,
    output: str,
    bzero: bool = False,
    no_bzero: bool = False,
    singleshell: bool = False,
    grad: InputPathType | None = None,
    fslgrad: DwiextractFslgradParamsDict | None = None,
    shells: list[float] | None = None,
    export_grad_mrtrix: str | None = None,
    export_grad_fsl: DwiextractExportGradFslParamsDict | None = None,
    import_pe_table: InputPathType | None = None,
    import_pe_eddy: DwiextractImportPeEddyParamsDict | None = None,
    pe: list[float] | None = None,
    strides: typing.Union[DwiextractVariousStringParamsDictTagged, DwiextractVariousFileParamsDictTagged] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DwiextractConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> DwiextractParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input DW image.
        output: the output image (diffusion-weighted volumes by default).
        bzero: Output b=0 volumes (instead of the diffusion weighted volumes,\
            if -singleshell is not specified).
        no_bzero: Output only non b=0 volumes (default, if -singleshell is not\
            specified).
        singleshell: Force a single-shell (single non b=0 shell) output. This\
            will include b=0 volumes, if present. Use with -bzero to enforce\
            presence of b=0 volumes (error if not present) or with -no_bzero to\
            exclude them.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        shells: specify one or more b-values to use during processing, as a\
            comma-separated list of the desired approximate b-values (b-values are\
            clustered to allow for small deviations). Note that some commands are\
            incompatible with multiple b-values, and will report an error if more\
            than one b-value is provided.\
            WARNING: note that, even though the b=0 volumes are never referred\
            to as shells in the literature, they still have to be explicitly\
            included in the list of b-values as provided to the -shell option!\
            Several algorithms which include the b=0 volumes in their\
            computations may otherwise return an undesired result.
        export_grad_mrtrix: export the diffusion-weighted gradient table to\
            file in MRtrix format.
        export_grad_fsl: export the diffusion-weighted gradient table to files\
            in FSL (bvecs / bvals) format.
        import_pe_table: import a phase-encoding table from file.
        import_pe_eddy: import phase-encoding information from an EDDY-style\
            config / index file pair.
        pe: select volumes with a particular phase encoding; this can be three\
            comma-separated values (for i,j,k components of vector direction) or\
            four (direction & total readout time).
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dwiextract",
        "bzero": bzero,
        "no_bzero": no_bzero,
        "singleshell": singleshell,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "output": output,
    }
    if grad is not None:
        params["grad"] = grad
    if fslgrad is not None:
        params["fslgrad"] = fslgrad
    if shells is not None:
        params["shells"] = shells
    if export_grad_mrtrix is not None:
        params["export_grad_mrtrix"] = export_grad_mrtrix
    if export_grad_fsl is not None:
        params["export_grad_fsl"] = export_grad_fsl
    if import_pe_table is not None:
        params["import_pe_table"] = import_pe_table
    if import_pe_eddy is not None:
        params["import_pe_eddy"] = import_pe_eddy
    if pe is not None:
        params["pe"] = pe
    if strides is not None:
        params["strides"] = strides
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dwiextract_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwiextractParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bzero", False) is None:
        raise StyxValidationError("`bzero` must not be None")
    if not isinstance(params["bzero"], bool):
        raise StyxValidationError(f'`bzero` has the wrong type: Received `{type(params.get("bzero", False))}` expected `bool`')
    if params.get("no_bzero", False) is None:
        raise StyxValidationError("`no_bzero` must not be None")
    if not isinstance(params["no_bzero"], bool):
        raise StyxValidationError(f'`no_bzero` has the wrong type: Received `{type(params.get("no_bzero", False))}` expected `bool`')
    if params.get("singleshell", False) is None:
        raise StyxValidationError("`singleshell` must not be None")
    if not isinstance(params["singleshell"], bool):
        raise StyxValidationError(f'`singleshell` has the wrong type: Received `{type(params.get("singleshell", False))}` expected `bool`')
    if params.get("grad", None) is not None:
        if not isinstance(params["grad"], (pathlib.Path, str)):
            raise StyxValidationError(f'`grad` has the wrong type: Received `{type(params.get("grad", None))}` expected `InputPathType | None`')
    if params.get("fslgrad", None) is not None:
        dwiextract_fslgrad_validate(params["fslgrad"])
    if params.get("shells", None) is not None:
        if not isinstance(params["shells"], list):
            raise StyxValidationError(f'`shells` has the wrong type: Received `{type(params.get("shells", None))}` expected `list[float] | None`')
        for e in params["shells"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`shells` has the wrong type: Received `{type(params.get("shells", None))}` expected `list[float] | None`')
    if params.get("export_grad_mrtrix", None) is not None:
        if not isinstance(params["export_grad_mrtrix"], str):
            raise StyxValidationError(f'`export_grad_mrtrix` has the wrong type: Received `{type(params.get("export_grad_mrtrix", None))}` expected `str | None`')
    if params.get("export_grad_fsl", None) is not None:
        dwiextract_export_grad_fsl_validate(params["export_grad_fsl"])
    if params.get("import_pe_table", None) is not None:
        if not isinstance(params["import_pe_table"], (pathlib.Path, str)):
            raise StyxValidationError(f'`import_pe_table` has the wrong type: Received `{type(params.get("import_pe_table", None))}` expected `InputPathType | None`')
    if params.get("import_pe_eddy", None) is not None:
        dwiextract_import_pe_eddy_validate(params["import_pe_eddy"])
    if params.get("pe", None) is not None:
        if not isinstance(params["pe"], list):
            raise StyxValidationError(f'`pe` has the wrong type: Received `{type(params.get("pe", None))}` expected `list[float] | None`')
        for e in params["pe"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`pe` has the wrong type: Received `{type(params.get("pe", None))}` expected `list[float] | None`')
    if params.get("strides", None) is not None:
        if not isinstance(params["strides"], dict):
            raise StyxValidationError(f'Params object has the wrong type \'{type(params["strides"])}\'')
        if "@type" not in params["strides"]:
            raise StyxValidationError("Params object is missing `@type`")
        if params["strides"]["@type"] not in ["VariousString", "VariousFile"]:
            raise StyxValidationError("Parameter `strides`s `@type` must be one of [\"VariousString\", \"VariousFile\"]")
        dwiextract_strides_validate_dyn_fn(params["strides"]["@type"])(params["strides"])
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[DwiextractConfigParamsDict] | None`')
        for e in params["config"]:
            dwiextract_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def dwiextract_cargs(
    params: DwiextractParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dwiextract")
    if params.get("bzero", False):
        cargs.append("-bzero")
    if params.get("no_bzero", False):
        cargs.append("-no_bzero")
    if params.get("singleshell", False):
        cargs.append("-singleshell")
    if params.get("grad", None) is not None:
        cargs.extend([
            "-grad",
            execution.input_file(params.get("grad", None))
        ])
    if params.get("fslgrad", None) is not None:
        cargs.extend(dwiextract_fslgrad_cargs(params.get("fslgrad", None), execution))
    if params.get("shells", None) is not None:
        cargs.extend([
            "-shells",
            ",".join(map(str, params.get("shells", None)))
        ])
    if params.get("export_grad_mrtrix", None) is not None:
        cargs.extend([
            "-export_grad_mrtrix",
            params.get("export_grad_mrtrix", None)
        ])
    if params.get("export_grad_fsl", None) is not None:
        cargs.extend(dwiextract_export_grad_fsl_cargs(params.get("export_grad_fsl", None), execution))
    if params.get("import_pe_table", None) is not None:
        cargs.extend([
            "-import_pe_table",
            execution.input_file(params.get("import_pe_table", None))
        ])
    if params.get("import_pe_eddy", None) is not None:
        cargs.extend(dwiextract_import_pe_eddy_cargs(params.get("import_pe_eddy", None), execution))
    if params.get("pe", None) is not None:
        cargs.extend([
            "-pe",
            ",".join(map(str, params.get("pe", None)))
        ])
    if params.get("strides", None) is not None:
        cargs.extend([
            "-strides",
            *dwiextract_strides_cargs_dyn_fn(params.get("strides", None)["@type"])(params.get("strides", None), execution)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dwiextract_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    return cargs


def dwiextract_outputs(
    params: DwiextractParamsDict,
    execution: Execution,
) -> DwiextractOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DwiextractOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
        export_grad_mrtrix=execution.output_file(params.get("export_grad_mrtrix", None)) if (params.get("export_grad_mrtrix") is not None) else None,
        export_grad_fsl=dwiextract_export_grad_fsl_outputs(params.get("export_grad_fsl"), execution) if params.get("export_grad_fsl") else None,
    )
    return ret


def dwiextract_execute(
    params: DwiextractParamsDict,
    runner: Runner | None = None,
) -> DwiextractOutputs:
    """
    dwiextract
    
    Extract diffusion-weighted volumes, b=0 volumes, or certain shells from a
    DWI dataset.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DwiextractOutputs`).
    """
    dwiextract_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DWIEXTRACT_METADATA)
    params = execution.params(params)
    cargs = dwiextract_cargs(params, execution)
    ret = dwiextract_outputs(params, execution)
    execution.run(cargs)
    return ret


def dwiextract(
    input_: InputPathType,
    output: str,
    bzero: bool = False,
    no_bzero: bool = False,
    singleshell: bool = False,
    grad: InputPathType | None = None,
    fslgrad: DwiextractFslgradParamsDict | None = None,
    shells: list[float] | None = None,
    export_grad_mrtrix: str | None = None,
    export_grad_fsl: DwiextractExportGradFslParamsDict | None = None,
    import_pe_table: InputPathType | None = None,
    import_pe_eddy: DwiextractImportPeEddyParamsDict | None = None,
    pe: list[float] | None = None,
    strides: typing.Union[DwiextractVariousStringParamsDictTagged, DwiextractVariousFileParamsDictTagged] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DwiextractConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DwiextractOutputs:
    """
    dwiextract
    
    Extract diffusion-weighted volumes, b=0 volumes, or certain shells from a
    DWI dataset.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input DW image.
        output: the output image (diffusion-weighted volumes by default).
        bzero: Output b=0 volumes (instead of the diffusion weighted volumes,\
            if -singleshell is not specified).
        no_bzero: Output only non b=0 volumes (default, if -singleshell is not\
            specified).
        singleshell: Force a single-shell (single non b=0 shell) output. This\
            will include b=0 volumes, if present. Use with -bzero to enforce\
            presence of b=0 volumes (error if not present) or with -no_bzero to\
            exclude them.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        shells: specify one or more b-values to use during processing, as a\
            comma-separated list of the desired approximate b-values (b-values are\
            clustered to allow for small deviations). Note that some commands are\
            incompatible with multiple b-values, and will report an error if more\
            than one b-value is provided.\
            WARNING: note that, even though the b=0 volumes are never referred\
            to as shells in the literature, they still have to be explicitly\
            included in the list of b-values as provided to the -shell option!\
            Several algorithms which include the b=0 volumes in their\
            computations may otherwise return an undesired result.
        export_grad_mrtrix: export the diffusion-weighted gradient table to\
            file in MRtrix format.
        export_grad_fsl: export the diffusion-weighted gradient table to files\
            in FSL (bvecs / bvals) format.
        import_pe_table: import a phase-encoding table from file.
        import_pe_eddy: import phase-encoding information from an EDDY-style\
            config / index file pair.
        pe: select volumes with a particular phase encoding; this can be three\
            comma-separated values (for i,j,k components of vector direction) or\
            four (direction & total readout time).
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DwiextractOutputs`).
    """
    params = dwiextract_params(
        bzero=bzero,
        no_bzero=no_bzero,
        singleshell=singleshell,
        grad=grad,
        fslgrad=fslgrad,
        shells=shells,
        export_grad_mrtrix=export_grad_mrtrix,
        export_grad_fsl=export_grad_fsl,
        import_pe_table=import_pe_table,
        import_pe_eddy=import_pe_eddy,
        pe=pe,
        strides=strides,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        output=output,
    )
    return dwiextract_execute(params, runner)


__all__ = [
    "DWIEXTRACT_METADATA",
    "DwiextractConfigParamsDict",
    "DwiextractConfigParamsDictTagged",
    "DwiextractExportGradFslOutputs",
    "DwiextractExportGradFslParamsDict",
    "DwiextractExportGradFslParamsDictTagged",
    "DwiextractFslgradParamsDict",
    "DwiextractFslgradParamsDictTagged",
    "DwiextractImportPeEddyParamsDict",
    "DwiextractImportPeEddyParamsDictTagged",
    "DwiextractOutputs",
    "DwiextractParamsDict",
    "DwiextractParamsDictTagged",
    "DwiextractVariousFileParamsDict",
    "DwiextractVariousFileParamsDictTagged",
    "DwiextractVariousStringParamsDict",
    "DwiextractVariousStringParamsDictTagged",
    "dwiextract",
    "dwiextract_config",
    "dwiextract_execute",
    "dwiextract_export_grad_fsl",
    "dwiextract_fslgrad",
    "dwiextract_import_pe_eddy",
    "dwiextract_params",
    "dwiextract_various_file",
    "dwiextract_various_string",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DWI2TENSOR_METADATA = Metadata(
    id="700cecd2ddde8992557eb31468df42097fa6e2b1.boutiques",
    name="dwi2tensor",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Dwi2tensorFslgradParamsDictNoTag = typing.TypedDict('_Dwi2tensorFslgradParamsDictNoTag', {
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
Dwi2tensorFslgradParamsDictTagged = typing.TypedDict('Dwi2tensorFslgradParamsDictTagged', {
    "@type": typing.Literal["fslgrad"],
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
Dwi2tensorFslgradParamsDict = _Dwi2tensorFslgradParamsDictNoTag | Dwi2tensorFslgradParamsDictTagged


_Dwi2tensorConfigParamsDictNoTag = typing.TypedDict('_Dwi2tensorConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Dwi2tensorConfigParamsDictTagged = typing.TypedDict('Dwi2tensorConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Dwi2tensorConfigParamsDict = _Dwi2tensorConfigParamsDictNoTag | Dwi2tensorConfigParamsDictTagged


_Dwi2tensorParamsDictNoTag = typing.TypedDict('_Dwi2tensorParamsDictNoTag', {
    "ols": bool,
    "mask": typing.NotRequired[InputPathType | None],
    "b0": typing.NotRequired[str | None],
    "dkt": typing.NotRequired[str | None],
    "iter": typing.NotRequired[int | None],
    "predicted_signal": typing.NotRequired[str | None],
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[Dwi2tensorFslgradParamsDict | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Dwi2tensorConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "dwi": InputPathType,
    "dt": str,
})
Dwi2tensorParamsDictTagged = typing.TypedDict('Dwi2tensorParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dwi2tensor"],
    "ols": bool,
    "mask": typing.NotRequired[InputPathType | None],
    "b0": typing.NotRequired[str | None],
    "dkt": typing.NotRequired[str | None],
    "iter": typing.NotRequired[int | None],
    "predicted_signal": typing.NotRequired[str | None],
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[Dwi2tensorFslgradParamsDict | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Dwi2tensorConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "dwi": InputPathType,
    "dt": str,
})
Dwi2tensorParamsDict = _Dwi2tensorParamsDictNoTag | Dwi2tensorParamsDictTagged


def dwi2tensor_fslgrad(
    bvecs: InputPathType,
    bvals: InputPathType,
) -> Dwi2tensorFslgradParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvals: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fslgrad",
        "bvecs": bvecs,
        "bvals": bvals,
    }
    return params


def dwi2tensor_fslgrad_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2tensorFslgradParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs", None) is None:
        raise StyxValidationError("`bvecs` must not be None")
    if not isinstance(params["bvecs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvecs` has the wrong type: Received `{type(params.get("bvecs", None))}` expected `InputPathType`')
    if params.get("bvals", None) is None:
        raise StyxValidationError("`bvals` must not be None")
    if not isinstance(params["bvals"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvals` has the wrong type: Received `{type(params.get("bvals", None))}` expected `InputPathType`')


def dwi2tensor_fslgrad_cargs(
    params: Dwi2tensorFslgradParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fslgrad")
    cargs.append(execution.input_file(params.get("bvecs", None)))
    cargs.append(execution.input_file(params.get("bvals", None)))
    return cargs


def dwi2tensor_config(
    key: str,
    value: str,
) -> Dwi2tensorConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dwi2tensor_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2tensorConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dwi2tensor_config_cargs(
    params: Dwi2tensorConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Dwi2tensorOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Dwi2tensorParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    dt: OutputPathType
    """the output dt image."""
    b0: OutputPathType | None
    """the output b0 image. """
    dkt: OutputPathType | None
    """the output dkt image. """
    predicted_signal: OutputPathType | None
    """the predicted dwi image. """


def dwi2tensor_params(
    dwi: InputPathType,
    dt: str,
    ols: bool = False,
    mask: InputPathType | None = None,
    b0: str | None = None,
    dkt: str | None = None,
    iter_: int | None = None,
    predicted_signal: str | None = None,
    grad: InputPathType | None = None,
    fslgrad: Dwi2tensorFslgradParamsDict | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Dwi2tensorConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Dwi2tensorParamsDictTagged:
    """
    Build parameters.
    
    Args:
        dwi: the input dwi image.
        dt: the output dt image.
        ols: perform initial fit using an ordinary least-squares (OLS) fit (see\
            Description).
        mask: only perform computation within the specified binary brain mask\
            image.
        b0: the output b0 image.
        dkt: the output dkt image.
        iter_: number of iterative reweightings for IWLS algorithm (default: 2)\
            (see Description).
        predicted_signal: the predicted dwi image.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dwi2tensor",
        "ols": ols,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "dwi": dwi,
        "dt": dt,
    }
    if mask is not None:
        params["mask"] = mask
    if b0 is not None:
        params["b0"] = b0
    if dkt is not None:
        params["dkt"] = dkt
    if iter_ is not None:
        params["iter"] = iter_
    if predicted_signal is not None:
        params["predicted_signal"] = predicted_signal
    if grad is not None:
        params["grad"] = grad
    if fslgrad is not None:
        params["fslgrad"] = fslgrad
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dwi2tensor_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Dwi2tensorParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("ols", False) is None:
        raise StyxValidationError("`ols` must not be None")
    if not isinstance(params["ols"], bool):
        raise StyxValidationError(f'`ols` has the wrong type: Received `{type(params.get("ols", False))}` expected `bool`')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("b0", None) is not None:
        if not isinstance(params["b0"], str):
            raise StyxValidationError(f'`b0` has the wrong type: Received `{type(params.get("b0", None))}` expected `str | None`')
    if params.get("dkt", None) is not None:
        if not isinstance(params["dkt"], str):
            raise StyxValidationError(f'`dkt` has the wrong type: Received `{type(params.get("dkt", None))}` expected `str | None`')
    if params.get("iter", None) is not None:
        if not isinstance(params["iter"], int):
            raise StyxValidationError(f'`iter` has the wrong type: Received `{type(params.get("iter", None))}` expected `int | None`')
    if params.get("predicted_signal", None) is not None:
        if not isinstance(params["predicted_signal"], str):
            raise StyxValidationError(f'`predicted_signal` has the wrong type: Received `{type(params.get("predicted_signal", None))}` expected `str | None`')
    if params.get("grad", None) is not None:
        if not isinstance(params["grad"], (pathlib.Path, str)):
            raise StyxValidationError(f'`grad` has the wrong type: Received `{type(params.get("grad", None))}` expected `InputPathType | None`')
    if params.get("fslgrad", None) is not None:
        dwi2tensor_fslgrad_validate(params["fslgrad"])
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Dwi2tensorConfigParamsDict] | None`')
        for e in params["config"]:
            dwi2tensor_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("dwi", None) is None:
        raise StyxValidationError("`dwi` must not be None")
    if not isinstance(params["dwi"], (pathlib.Path, str)):
        raise StyxValidationError(f'`dwi` has the wrong type: Received `{type(params.get("dwi", None))}` expected `InputPathType`')
    if params.get("dt", None) is None:
        raise StyxValidationError("`dt` must not be None")
    if not isinstance(params["dt"], str):
        raise StyxValidationError(f'`dt` has the wrong type: Received `{type(params.get("dt", None))}` expected `str`')


def dwi2tensor_cargs(
    params: Dwi2tensorParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dwi2tensor")
    if params.get("ols", False):
        cargs.append("-ols")
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("b0", None) is not None:
        cargs.extend([
            "-b0",
            params.get("b0", None)
        ])
    if params.get("dkt", None) is not None:
        cargs.extend([
            "-dkt",
            params.get("dkt", None)
        ])
    if params.get("iter", None) is not None:
        cargs.extend([
            "-iter",
            str(params.get("iter", None))
        ])
    if params.get("predicted_signal", None) is not None:
        cargs.extend([
            "-predicted_signal",
            params.get("predicted_signal", None)
        ])
    if params.get("grad", None) is not None:
        cargs.extend([
            "-grad",
            execution.input_file(params.get("grad", None))
        ])
    if params.get("fslgrad", None) is not None:
        cargs.extend(dwi2tensor_fslgrad_cargs(params.get("fslgrad", None), execution))
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dwi2tensor_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("dwi", None)))
    cargs.append(params.get("dt", None))
    return cargs


def dwi2tensor_outputs(
    params: Dwi2tensorParamsDict,
    execution: Execution,
) -> Dwi2tensorOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Dwi2tensorOutputs(
        root=execution.output_file("."),
        dt=execution.output_file(params.get("dt", None)),
        b0=execution.output_file(params.get("b0", None)) if (params.get("b0") is not None) else None,
        dkt=execution.output_file(params.get("dkt", None)) if (params.get("dkt") is not None) else None,
        predicted_signal=execution.output_file(params.get("predicted_signal", None)) if (params.get("predicted_signal") is not None) else None,
    )
    return ret


def dwi2tensor_execute(
    params: Dwi2tensorParamsDict,
    runner: Runner | None = None,
) -> Dwi2tensorOutputs:
    """
    dwi2tensor
    
    Diffusion (kurtosis) tensor estimation.
    
    By default, the diffusion tensor (and optionally its kurtosis) is fitted to
    the log-signal in two steps: firstly, using weighted least-squares (WLS)
    with weights based on the empirical signal intensities; secondly, by further
    iterated weighted least-squares (IWLS) with weights determined by the signal
    predictions from the previous iteration (by default, 2 iterations will be
    performed). This behaviour can be altered in two ways:
    
    * The -ols option will cause the first fitting step to be performed using
    ordinary least-squares (OLS); that is, all measurements contribute equally
    to the fit, instead of the default behaviour of weighting based on the
    empirical signal intensities.
    
    * The -iter option controls the number of iterations of the IWLS prodedure.
    If this is set to zero, then the output model parameters will be those
    resulting from the first fitting step only: either WLS by default, or OLS if
    the -ols option is used in conjunction with -iter 0.
    
    The tensor coefficients are stored in the output image as follows:
    volumes 0-5: D11, D22, D33, D12, D13, D23
    
    If diffusion kurtosis is estimated using the -dkt option, these are stored
    as follows:
    volumes 0-2: W1111, W2222, W3333
    volumes 3-8: W1112, W1113, W1222, W1333, W2223, W2333
    volumes 9-11: W1122, W1133, W2233
    volumes 12-14: W1123, W1223, W1233
    
    References:
    
    References based on fitting algorithm used:
    
    * OLS, WLS:
    Basser, P.J.; Mattiello, J.; LeBihan, D. Estimation of the effective
    self-diffusion tensor from the NMR spin echo. J Magn Reson B., 1994, 103,
    247â€“254.
    
    * IWLS:
    Veraart, J.; Sijbers, J.; Sunaert, S.; Leemans, A. & Jeurissen, B. Weighted
    linear least squares estimation of diffusion MRI parameters: strengths,
    limitations, and pitfalls. NeuroImage, 2013, 81, 335-346.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Dwi2tensorOutputs`).
    """
    dwi2tensor_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DWI2TENSOR_METADATA)
    params = execution.params(params)
    cargs = dwi2tensor_cargs(params, execution)
    ret = dwi2tensor_outputs(params, execution)
    execution.run(cargs)
    return ret


def dwi2tensor(
    dwi: InputPathType,
    dt: str,
    ols: bool = False,
    mask: InputPathType | None = None,
    b0: str | None = None,
    dkt: str | None = None,
    iter_: int | None = None,
    predicted_signal: str | None = None,
    grad: InputPathType | None = None,
    fslgrad: Dwi2tensorFslgradParamsDict | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Dwi2tensorConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Dwi2tensorOutputs:
    """
    dwi2tensor
    
    Diffusion (kurtosis) tensor estimation.
    
    By default, the diffusion tensor (and optionally its kurtosis) is fitted to
    the log-signal in two steps: firstly, using weighted least-squares (WLS)
    with weights based on the empirical signal intensities; secondly, by further
    iterated weighted least-squares (IWLS) with weights determined by the signal
    predictions from the previous iteration (by default, 2 iterations will be
    performed). This behaviour can be altered in two ways:
    
    * The -ols option will cause the first fitting step to be performed using
    ordinary least-squares (OLS); that is, all measurements contribute equally
    to the fit, instead of the default behaviour of weighting based on the
    empirical signal intensities.
    
    * The -iter option controls the number of iterations of the IWLS prodedure.
    If this is set to zero, then the output model parameters will be those
    resulting from the first fitting step only: either WLS by default, or OLS if
    the -ols option is used in conjunction with -iter 0.
    
    The tensor coefficients are stored in the output image as follows:
    volumes 0-5: D11, D22, D33, D12, D13, D23
    
    If diffusion kurtosis is estimated using the -dkt option, these are stored
    as follows:
    volumes 0-2: W1111, W2222, W3333
    volumes 3-8: W1112, W1113, W1222, W1333, W2223, W2333
    volumes 9-11: W1122, W1133, W2233
    volumes 12-14: W1123, W1223, W1233
    
    References:
    
    References based on fitting algorithm used:
    
    * OLS, WLS:
    Basser, P.J.; Mattiello, J.; LeBihan, D. Estimation of the effective
    self-diffusion tensor from the NMR spin echo. J Magn Reson B., 1994, 103,
    247â€“254.
    
    * IWLS:
    Veraart, J.; Sijbers, J.; Sunaert, S.; Leemans, A. & Jeurissen, B. Weighted
    linear least squares estimation of diffusion MRI parameters: strengths,
    limitations, and pitfalls. NeuroImage, 2013, 81, 335-346.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        dwi: the input dwi image.
        dt: the output dt image.
        ols: perform initial fit using an ordinary least-squares (OLS) fit (see\
            Description).
        mask: only perform computation within the specified binary brain mask\
            image.
        b0: the output b0 image.
        dkt: the output dkt image.
        iter_: number of iterative reweightings for IWLS algorithm (default: 2)\
            (see Description).
        predicted_signal: the predicted dwi image.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Dwi2tensorOutputs`).
    """
    params = dwi2tensor_params(
        ols=ols,
        mask=mask,
        b0=b0,
        dkt=dkt,
        iter_=iter_,
        predicted_signal=predicted_signal,
        grad=grad,
        fslgrad=fslgrad,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        dwi=dwi,
        dt=dt,
    )
    return dwi2tensor_execute(params, runner)


__all__ = [
    "DWI2TENSOR_METADATA",
    "Dwi2tensorConfigParamsDict",
    "Dwi2tensorConfigParamsDictTagged",
    "Dwi2tensorFslgradParamsDict",
    "Dwi2tensorFslgradParamsDictTagged",
    "Dwi2tensorOutputs",
    "Dwi2tensorParamsDict",
    "Dwi2tensorParamsDictTagged",
    "dwi2tensor",
    "dwi2tensor_config",
    "dwi2tensor_execute",
    "dwi2tensor_fslgrad",
    "dwi2tensor_params",
]
